package com.db;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import com.model.KeyValuePair;
import com.model.DosboxVersion;
import com.model.Profile;
import com.model.Settings;
import com.model.Template;
import com.util.FileUtils;
import com.gui.GeneralPurposeDialogs;
import org.eclipse.swt.widgets.Shell;


public class Database {

    private Connection con = null;

    private static final String GET_IDENTITY_QUERY = "CALL IDENTITY()";
    private static final String GAME_LIST_QUERY = "SELECT GAM.ID, GAM.TITLE, DEV.NAME, PUBL.NAME, GEN.NAME, YR.YEAR, GAM.DBVERSION_ID, GAM.SETUP, "
            + "GAM.SETUP_PARAMS, GAM.NOTES, GAM.LINK1, GAM.LINK2, GAM.FAVORITE, STAT.STAT, GAM.CONFFILE, GAM.CAPTURES, GAM.LINK3, GAM.LINK4, "
            + "CUST1.VALUE, CUST2.VALUE, CUST3.VALUE, CUST4.VALUE, GAM.CUSTOM5, GAM.CUSTOM6, GAM.CUSTOM7, GAM.CUSTOM8, GAM.CUSTOM9, GAM.CUSTOM10 "
            + "FROM GAMES GAM, DEVELOPERS DEV, PUBLISHERS PUBL, GENRES GEN, PUBLYEARS YR, STATUS STAT, "
            + "CUSTOM1 CUST1, CUSTOM2 CUST2, CUSTOM3 CUST3, CUSTOM4 CUST4 "
            + "WHERE GAM.DEV_ID=DEV.ID AND GAM.PUBL_ID=PUBL.ID AND GAM.GENRE_ID=GEN.ID AND GAM.YEAR_ID=YR.ID AND GAM.STAT_ID=STAT.ID "
            + "AND   GAM.CUST1_ID=CUST1.ID AND GAM.CUST2_ID=CUST2.ID AND GAM.CUST3_ID=CUST3.ID AND GAM.CUST4_ID=CUST4.ID "
            + "ORDER BY ";
    private static final String DUPLICATE_GAME_QUERY = "INSERT INTO GAMES("
            + "TITLE, DEV_ID, PUBL_ID, GENRE_ID, YEAR_ID, DBVERSION_ID, STAT_ID, SETUP, SETUP_PARAMS, NOTES,"
            + "LINK1, LINK2, FAVORITE, CONFFILE, CAPTURES, LINK3, LINK4, CUST1_ID, CUST2_ID, CUST3_ID, CUST4_ID,"
            + "CUSTOM5, CUSTOM6, CUSTOM7, CUSTOM8, CUSTOM9, CUSTOM10) "
            + "(SELECT TITLE, DEV_ID, PUBL_ID, GENRE_ID, YEAR_ID, DBVERSION_ID, STAT_ID, SETUP, SETUP_PARAMS, NOTES,"
            + "LINK1, LINK2, FAVORITE, NULL,     NULL,     LINK3, LINK4, CUST1_ID, CUST2_ID, CUST3_ID, CUST4_ID,"
            + "CUSTOM5, CUSTOM6, CUSTOM7, CUSTOM8, CUSTOM9, CUSTOM10" + " FROM GAMES WHERE ID = ?)";
    private static final String DEVELOPER_LIST_QUERY = "SELECT ID, NAME FROM DEVELOPERS ORDER BY NAME";
    private static final String PUBLISHER_LIST_QUERY = "SELECT ID, NAME FROM PUBLISHERS ORDER BY NAME";
    private static final String GENRE_LIST_QUERY = "SELECT ID, NAME FROM GENRES ORDER BY NAME";
    private static final String PUBLYEAR_LIST_QUERY = "SELECT ID, YEAR FROM PUBLYEARS ORDER BY YEAR";
    private static final String DOSBOXVERSIONS_LIST_QUERY = "SELECT ID, TITLE, PATH, MULTICONF, DEFAULT, PARAMETERS, VERSION FROM DOSBOXVERSIONS ORDER BY TITLE";
    private static final String TEMPLATES_LIST_QUERY = "SELECT ID, TITLE, DBVERSION_ID, DEFAULT FROM TEMPLATES ORDER BY TITLE";
    private static final String STATUS_LIST_QUERY = "SELECT ID, STAT FROM STATUS ORDER BY STAT";
    private static final String[] CUSTOM_LIST_QUERY = { "SELECT ID, VALUE FROM CUSTOM1 ORDER BY VALUE",
            "SELECT ID, VALUE FROM CUSTOM2 ORDER BY VALUE", "SELECT ID, VALUE FROM CUSTOM3 ORDER BY VALUE",
            "SELECT ID, VALUE FROM CUSTOM4 ORDER BY VALUE" };

    private static final String ADD_GAME_QUERY = "INSERT INTO GAMES(TITLE, DEV_ID, PUBL_ID, GENRE_ID, YEAR_ID, DBVERSION_ID, STAT_ID, SETUP, "
            + "SETUP_PARAMS, NOTES, LINK1, LINK2, FAVORITE, CONFFILE, CAPTURES, LINK3, LINK4, CUST1_ID, CUST2_ID, CUST3_ID, CUST4_ID,"
            + "CUSTOM5, CUSTOM6, CUSTOM7, CUSTOM8, CUSTOM9, CUSTOM10) "
            + "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, NULL, NULL, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
    private static final String ADD_DEVELOPER_QUERY = "INSERT INTO DEVELOPERS(NAME) VALUES (?)";
    private static final String ADD_PUBLISHER_QUERY = "INSERT INTO PUBLISHERS(NAME) VALUES (?)";
    private static final String ADD_GENRE_QUERY = "INSERT INTO GENRES(NAME) VALUES (?)";
    private static final String ADD_YEAR_QUERY = "INSERT INTO PUBLYEARS(YEAR) VALUES (?)";
    private static final String ADD_DOSBOXVERSION_QUERY = "INSERT INTO DOSBOXVERSIONS(TITLE, PATH, MULTICONF, DEFAULT, PARAMETERS, VERSION) VALUES (?, ?, ?, ?, ?, ?)";
    private static final String ADD_TEMPLATE_QUERY = "INSERT INTO TEMPLATES(TITLE, DBVERSION_ID, DEFAULT) VALUES (?, ?, ?)";
    private static final String ADD_STATUS_QUERY = "INSERT INTO STATUS(STAT) VALUES (?)";
    private static final String[] ADD_CUSTOM_QUERY = { "INSERT INTO CUSTOM1(VALUE) VALUES (?)",
            "INSERT INTO CUSTOM2(VALUE) VALUES (?)", "INSERT INTO CUSTOM3(VALUE) VALUES (?)",
            "INSERT INTO CUSTOM4(VALUE) VALUES (?)" };

    private static final String UPDATE_GAME_QUERY = "UPDATE GAMES SET TITLE = ?, DEV_ID = ?, PUBL_ID = ?, GENRE_ID = ?, YEAR_ID = ?, "
            + "DBVERSION_ID = ?, STAT_ID = ?, SETUP = ?, SETUP_PARAMS = ?, NOTES = ?, LINK1 = ?, LINK2 = ?, FAVORITE = ?, "
            + "LINK3 = ?, LINK4 = ?, CUST1_ID = ?, CUST2_ID = ?, CUST3_ID = ?, CUST4_ID = ?, "
            + "CUSTOM5 = ?, CUSTOM6 = ?, CUSTOM7 = ?, CUSTOM8 = ?, CUSTOM9 = ?, CUSTOM10 = ?  " + "WHERE ID = ?";
    private static final String UPDATE_GAME_CONF_QUERY = "UPDATE GAMES SET CONFFILE = ?, CAPTURES = ? WHERE ID = ?";
    private static final String UPDATE_GAME_SETUP_AND_LINKS_QUERY = "UPDATE GAMES SET SETUP = ?, LINK1 = ?, LINK2 = ?, LINK3 = ?, LINK4 = ? WHERE ID = ?";
    private static final String UPDATE_DOSBOXVERSION_NODEFAULT = "UPDATE DOSBOXVERSIONS SET DEFAULT = FALSE";
    private static final String UPDATE_DOSBOXVERSION_QUERY = "UPDATE DOSBOXVERSIONS SET TITLE = ?, PATH = ?, MULTICONF = ?, DEFAULT = ?, PARAMETERS = ?, VERSION = ? WHERE ID = ?";
    private static final String UPDATE_TEMPLATE_QUERY = "UPDATE TEMPLATES SET TITLE = ?, DBVERSION_ID = ?, DEFAULT = ? WHERE ID = ?";
    private static final String UPDATE_TEMPLATE_NODEFAULT = "UPDATE TEMPLATES SET DEFAULT = FALSE";

    private static final String REMOVE_GAME_QUERY = "DELETE FROM GAMES WHERE ID = ?";
    private static final String REMOVE_DOSBOXVERSION_QUERY = "DELETE FROM DOSBOXVERSIONS WHERE ID = ?";
    private static final String REMOVE_TEMPLATE_QUERY = "DELETE FROM TEMPLATES WHERE ID = ?";

    private static final String TOGGLE_FAVORITE_QUERY = "UPDATE GAMES SET FAVORITE = NOT FAVORITE WHERE ID = ?";

    private static final String GET_VERSION = "SELECT MAJORVERSION, MINORVERSION FROM VERSION";
    private static final String UPGRADE_TO_V050_QUERY = "ALTER TABLE GAMES ADD COLUMN CONFFILE VARCHAR(256);"
            + "ALTER TABLE GAMES ADD COLUMN CAPTURES VARCHAR(256);"
            + "CREATE TABLE VERSION(MAJORVERSION INTEGER NOT NULL, MINORVERSION INTEGER NOT NULL);"
            + "INSERT INTO VERSION VALUES(0, 50);" + "UPDATE GAMES SET" + " CAPTURES = '" + FileUtils.CAPTURES_DIR
            + "' || GAMES.ID," + " CONFFILE = '" + FileUtils.PROFILES_DIR + "' || GAMES.ID || '" + FileUtils.CONF_EXT + "';";
    private static final String UPGRADE_TO_V051_QUERY = "ALTER TABLE DOSBOXVERSIONS ADD COLUMN PARAMETERS VARCHAR(256) DEFAULT '';"
            + "UPDATE VERSION SET MINORVERSION = 51;";
    private static final String UPGRADE_TO_V056_QUERY = "ALTER TABLE GAMES ADD COLUMN LINK3 VARCHAR(256) DEFAULT '';"
            + "ALTER TABLE GAMES ADD COLUMN LINK4 VARCHAR(256) DEFAULT '';"
            + "ALTER TABLE GAMES ADD COLUMN CUST1_ID INTEGER DEFAULT 0;"
            + "ALTER TABLE GAMES ADD COLUMN CUST2_ID INTEGER DEFAULT 0;"
            + "ALTER TABLE GAMES ADD COLUMN CUST3_ID INTEGER DEFAULT 0;"
            + "ALTER TABLE GAMES ADD COLUMN CUST4_ID INTEGER DEFAULT 0;"
            + "ALTER TABLE GAMES ADD COLUMN CUSTOM5 VARCHAR(256) DEFAULT '';"
            + "ALTER TABLE GAMES ADD COLUMN CUSTOM6 VARCHAR(256) DEFAULT '';"
            + "ALTER TABLE GAMES ADD COLUMN CUSTOM7 VARCHAR(256) DEFAULT '';"
            + "ALTER TABLE GAMES ADD COLUMN CUSTOM8 VARCHAR(256) DEFAULT '';"
            + "ALTER TABLE GAMES ADD COLUMN CUSTOM9  INTEGER DEFAULT 0;"
            + "ALTER TABLE GAMES ADD COLUMN CUSTOM10 INTEGER DEFAULT 0;"
            + "CREATE TABLE CUSTOM1(ID INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 0) NOT NULL PRIMARY KEY, VALUE VARCHAR(256) NOT NULL);"
            + "CREATE TABLE CUSTOM2(ID INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 0) NOT NULL PRIMARY KEY, VALUE VARCHAR(256) NOT NULL);"
            + "CREATE TABLE CUSTOM3(ID INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 0) NOT NULL PRIMARY KEY, VALUE VARCHAR(256) NOT NULL);"
            + "CREATE TABLE CUSTOM4(ID INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 0) NOT NULL PRIMARY KEY, VALUE VARCHAR(256) NOT NULL);"
            + "INSERT INTO CUSTOM1(VALUE) VALUES('');" + "INSERT INTO CUSTOM2(VALUE) VALUES('');"
            + "INSERT INTO CUSTOM3(VALUE) VALUES('');" + "INSERT INTO CUSTOM4(VALUE) VALUES('');"
            + "UPDATE VERSION SET MINORVERSION = 56;";
    static final String UPGRADE_TO_V062_QUERY = "ALTER TABLE DOSBOXVERSIONS ADD COLUMN VERSION VARCHAR(256) DEFAULT '0.72' NOT NULL;"
            + "UPDATE VERSION SET MINORVERSION = 62;";
    private static final String CREATE_INITIAL_TABLES = "SET WRITE_DELAY 1;"
            + "CREATE MEMORY TABLE VERSION(MAJORVERSION INTEGER NOT NULL,MINORVERSION INTEGER NOT NULL);"
            + "CREATE MEMORY TABLE DEVELOPERS(ID INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 0) NOT NULL PRIMARY KEY,NAME VARCHAR(256) NOT NULL);"
            + "CREATE MEMORY TABLE PUBLISHERS(ID INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 0) NOT NULL PRIMARY KEY,NAME VARCHAR(256) NOT NULL);"
            + "CREATE MEMORY TABLE GENRES(ID INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 0) NOT NULL PRIMARY KEY,NAME VARCHAR(256) NOT NULL);"
            + "CREATE MEMORY TABLE PUBLYEARS(ID INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 0) NOT NULL PRIMARY KEY,YEAR CHAR(4) NOT NULL);"
            + "CREATE MEMORY TABLE STATUS(ID INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 0) NOT NULL PRIMARY KEY,STAT VARCHAR(256) NOT NULL);"
            + "CREATE MEMORY TABLE CUSTOM1(ID INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 0) NOT NULL PRIMARY KEY,VALUE VARCHAR(256) NOT NULL);"
            + "CREATE MEMORY TABLE CUSTOM2(ID INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 0) NOT NULL PRIMARY KEY,VALUE VARCHAR(256) NOT NULL);"
            + "CREATE MEMORY TABLE CUSTOM3(ID INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 0) NOT NULL PRIMARY KEY,VALUE VARCHAR(256) NOT NULL);"
            + "CREATE MEMORY TABLE CUSTOM4(ID INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 0) NOT NULL PRIMARY KEY,VALUE VARCHAR(256) NOT NULL);"
            + "CREATE MEMORY TABLE DOSBOXVERSIONS(ID INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 0) NOT NULL PRIMARY KEY,TITLE VARCHAR(256) NOT NULL,PATH VARCHAR(256) NOT NULL,MULTICONF BOOLEAN,DEFAULT BOOLEAN,PARAMETERS VARCHAR(256) DEFAULT '',VERSION VARCHAR(256) NOT NULL);"
            + "CREATE MEMORY TABLE TEMPLATES(ID INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 0) NOT NULL PRIMARY KEY,TITLE VARCHAR(256) NOT NULL,DBVERSION_ID INTEGER,DEFAULT BOOLEAN,CONSTRAINT SYS_FK_185 FOREIGN KEY(DBVERSION_ID) REFERENCES DOSBOXVERSIONS(ID));"
            + "CREATE MEMORY TABLE GAMES(ID INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 0) NOT NULL PRIMARY KEY,TITLE VARCHAR(256) NOT NULL,DEV_ID INTEGER,PUBL_ID INTEGER,GENRE_ID INTEGER,YEAR_ID INTEGER,DBVERSION_ID INTEGER,STAT_ID INTEGER,SETUP VARCHAR(256),SETUP_PARAMS VARCHAR(256),NOTES LONGVARCHAR,LINK1 VARCHAR(256),LINK2 VARCHAR(256),FAVORITE BOOLEAN,CONFFILE VARCHAR(256),CAPTURES VARCHAR(256),LINK3 VARCHAR(256) DEFAULT '',LINK4 VARCHAR(256) DEFAULT '',CUST1_ID INTEGER DEFAULT 0,CUST2_ID INTEGER DEFAULT 0,CUST3_ID INTEGER DEFAULT 0,CUST4_ID INTEGER DEFAULT 0,CUSTOM5 VARCHAR(256) DEFAULT '',CUSTOM6 VARCHAR(256) DEFAULT '',CUSTOM7 VARCHAR(256) DEFAULT '',CUSTOM8 VARCHAR(256) DEFAULT '',CUSTOM9 INTEGER DEFAULT 0,CUSTOM10 INTEGER DEFAULT 0,CONSTRAINT SYS_FK_165 FOREIGN KEY(DEV_ID) REFERENCES DEVELOPERS(ID),CONSTRAINT SYS_FK_166 FOREIGN KEY(PUBL_ID) REFERENCES PUBLISHERS(ID),CONSTRAINT SYS_FK_167 FOREIGN KEY(GENRE_ID) REFERENCES GENRES(ID),CONSTRAINT SYS_FK_168 FOREIGN KEY(YEAR_ID) REFERENCES PUBLYEARS(ID),CONSTRAINT SYS_FK_169 FOREIGN KEY(DBVERSION_ID) REFERENCES DOSBOXVERSIONS(ID),CONSTRAINT SYS_FK_170 FOREIGN KEY(STAT_ID) REFERENCES STATUS(ID));"
            + "INSERT INTO DOSBOXVERSIONS VALUES(0,'DOSBox 0.72','DOSBox-0.72',TRUE,TRUE,'','0.72');"
            + "INSERT INTO TEMPLATES VALUES(0,'Booter',0,FALSE);"
            + "INSERT INTO TEMPLATES VALUES(1,'Early 80''s (CGA, 640KB conv. memory, speaker, 8086 ~4.77Mhz)',0,FALSE);"
            + "INSERT INTO TEMPLATES VALUES(2,'Late  80''s (VGA, 640KB conv. memory, speaker, 80286 ~12Mhz)',0,FALSE);"
            + "INSERT INTO TEMPLATES VALUES(3,'Early 90''s (VGA, 4MB ram, full audio support, 80386 ~25Mhz)',0,FALSE);"
            + "INSERT INTO TEMPLATES VALUES(4,'Late  90''s (VGA, 16MB ram, full audio support, 80486 ~66Mhz)',0,FALSE);"
            + "INSERT INTO VERSION VALUES(0,62);";
    private static final String TEST_QUERY = "SELECT TOP 1 ID FROM GAMES";

    private static final int COMPLEX_CUSTOM_STRINGS = CUSTOM_LIST_QUERY.length;
    private static final int SIMPLE_CUSTOM_STRINGS = 4;
    private static final int SIMPLE_CUSTOM_INTS = 2;


    private Database() {
        init();
    }

    private static class DatabaseHolder {
        private static Database instance = new Database();
    }

    public static Database getInstance() {
        return DatabaseHolder.instance;
    }
    
    private void closeAll(Statement st, ResultSet rs) {
        if (st != null) try { st.close(); } catch (SQLException e) {}
        if (rs != null) try { rs.close(); } catch (SQLException e) {}
    }

    private int[] getVersion() {
        int[] result = new int[2];
        Statement stmt = null;
        ResultSet rs = null;
        try {
            stmt = con.createStatement();
            rs = stmt.executeQuery(GET_VERSION);
            if (rs != null && rs.next()) {
                result[0] = rs.getInt(1); // major
                result[1] = rs.getInt(2); // minor
            }
        } catch (SQLException e) {
            // assume version < 0.50 (0.0)
        } finally {
            closeAll(stmt, rs);
        }
        return result;
    }

    private void upgradeToVersion(String query, int minorVersion) throws SQLException {
        System.out.println("upgrading database to v0." + minorVersion);
        Statement stmt = null;
        try {
            stmt = con.createStatement();
            stmt.executeQuery(query);
        } catch (SQLException e) {
            e.printStackTrace();
            throw new SQLException("Could not upgrade the database to v0." + minorVersion);
        } finally {
            closeAll(stmt, null);
        }        
    }

    private void initializeIfNecessary() throws SQLException {
        Statement stmt = null;
        try {
            stmt = con.createStatement();
            stmt.executeQuery(TEST_QUERY);
        } catch (SQLException emptydatabase) {
            // Probably empty database, fill it
            Statement stmt2 = null;
            try {
                stmt2 = con.createStatement();
                stmt2.executeQuery(CREATE_INITIAL_TABLES);
            } catch (SQLException e) {
                e.printStackTrace();
                throw new SQLException("Database query 'create initial tables' failed");
            } finally {
                closeAll(stmt2, null);
            }
        } finally {
            closeAll(stmt, null);
        }
    }

    private void upgradeIfNecessary() throws SQLException {
        int[] version = getVersion();
        if (version[0] <= 0 && version[1] < 50) upgradeToVersion(UPGRADE_TO_V050_QUERY, 50);
        if (version[0] <= 0 && version[1] < 51) upgradeToVersion(UPGRADE_TO_V051_QUERY, 51);
        if (version[0] <= 0 && version[1] < 56) upgradeToVersion(UPGRADE_TO_V056_QUERY, 56);
        if (version[0] <= 0 && version[1] < 62) upgradeToVersion(UPGRADE_TO_V062_QUERY, 62);
    }

    private void init() {
        System.out.println("startup");
        Settings settings = Settings.getInstance();
        String connString = settings.getValue("database", "connectionstring");
        if (connString.contains("file:")) {
            // Some magic on the connection string
            int start = connString.indexOf("file:") + 5; // skip 'file:'
            int end = connString.indexOf(';', start);
            if (end == -1) end = connString.length();
            String databasefile = FileUtils.getCanonicalPath(connString.substring(start, end));
            connString = connString.substring(0, start) + databasefile + connString.substring(end);
        }
        try {
            // Register the JDBC driver for dBase
            Class.forName("org.hsqldb.jdbcDriver");
            con = DriverManager.getConnection(connString,
                    settings.getValue("database", "username"), settings.getValue("database", "password"));
            initializeIfNecessary();
            upgradeIfNecessary();
        } catch (SQLException e) {
            e.printStackTrace();
            try {
                if (con != null) con.close();
            } catch (SQLException exc) {
                System.err.println("JDBC Connection could not be closed properly!!\n");
                exc.printStackTrace();
            }
            Shell shell = new Shell();
            GeneralPurposeDialogs.fatalMessage(shell,
                    "There was a problem initializing the database connection!\n\nTechnical information:\n" + e.toString());
            System.exit(1);
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
            Shell shell = new Shell();
            GeneralPurposeDialogs.fatalMessage(shell,
                    "There was a problem registering the database JDBC driver!\n\nTechnical information:\n" + e.toString());
            System.exit(1);
        }
    }

    private int identity() throws SQLException {
        int result = -1;
        Statement stmt = null;
        ResultSet rs = null;
        try {
            stmt = con.createStatement();
            rs = stmt.executeQuery(GET_IDENTITY_QUERY);
            if (rs != null && rs.next()) result = rs.getInt(1);
        } catch (SQLException e) {
            e.printStackTrace();
            throw new SQLException("Database query 'get identity' failed");
        } finally {
            closeAll(stmt, rs);
        }
        return result;
    }

    private int addSomething(String query, String objectToAdd) throws SQLException {
        PreparedStatement stmt = null;
        try {
            stmt = con.prepareStatement(query);
            stmt.setString(1, objectToAdd);
            stmt.executeUpdate();
        } catch (SQLException e) {
            e.printStackTrace();
            throw new SQLException("Database query 'add something' failed");
        } finally {
            closeAll(stmt, null);
        }
        return identity();
    }
    
    private void removeSomething(String query, String objectToRemove, int objectId) throws SQLException {
        PreparedStatement stmt = null;
        try {
            stmt = con.prepareStatement(query);
            stmt.setInt(1, objectId);
            stmt.executeUpdate();
        } catch (SQLException e) {
            e.printStackTrace();
            throw new SQLException("Database query 'remove " + objectToRemove + "' failed");
        } finally {
            closeAll(stmt, null);
        }
    }

    private List<KeyValuePair> readKeyValuePairList(String query) throws SQLException {
        List<KeyValuePair> customList = new ArrayList<KeyValuePair>();
        Statement stmt = null;
        ResultSet rs = null;
        try {
            stmt = con.createStatement();
            rs = stmt.executeQuery(query);
            if (rs != null)
                while (rs.next())
                    customList.add(new KeyValuePair(rs.getInt(1), rs.getString(2)));
        } catch (SQLException e) {
            e.printStackTrace();
            throw new SQLException("Database query 'read KeyValuePairsList' failed");
        } finally {
            closeAll(stmt, rs);
        }
        return customList;
    }

    public void shutdown() throws SQLException {
        System.out.println("shutdown");
        Statement stmt = null;
        try {
            stmt = con.createStatement();
            stmt.execute("SHUTDOWN");
        } catch (SQLException e) {
            e.printStackTrace();
            throw new SQLException("Database connection could not be shutdown properly");
        } finally {
            closeAll(stmt, null);
            con.close();
        }
    }

    public List<Profile> readProfilesList(String orderingClause) throws SQLException {
        List<Profile> profilesList = new ArrayList<Profile>();
        Statement stmt = null;
        ResultSet rs = null;
        try {
            stmt = con.createStatement();
            rs = stmt.executeQuery(GAME_LIST_QUERY + orderingClause);
            if (rs != null) {
                while (rs.next()) {
                    profilesList.add(new Profile(rs.getInt(1), rs.getString(2), rs.getString(3), rs.getString(4),
                            rs.getString(5), rs.getString(6), rs.getInt(7), rs.getString(8), rs.getString(9),
                            rs.getString(10), rs.getString(11), rs.getString(12), rs.getBoolean(13), rs.getString(14),
                            rs.getString(15), rs.getString(16), rs.getString(17), rs.getString(18), rs.getString(19),
                            rs.getString(20), rs.getString(21), rs.getString(22), rs.getString(23), rs.getString(24),
                            rs.getString(25), rs.getString(26), rs.getInt(27), rs.getInt(28)));
                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
            throw new SQLException("Database query 'read profiles' failed");
        } finally {
            closeAll(stmt, rs);
        }
        return profilesList;
    }

    public List<KeyValuePair> readDevelopersList() throws SQLException {
        return readKeyValuePairList(DEVELOPER_LIST_QUERY);
    }

    public List<KeyValuePair> readPublishersList() throws SQLException {
        return readKeyValuePairList(PUBLISHER_LIST_QUERY);
    }

    public List<KeyValuePair> readGenresList() throws SQLException {
        return readKeyValuePairList(GENRE_LIST_QUERY);
    }

    public List<KeyValuePair> readYearsList() throws SQLException {
        return readKeyValuePairList(PUBLYEAR_LIST_QUERY);
    }

    public List<KeyValuePair> readStatusList() throws SQLException {
        return readKeyValuePairList(STATUS_LIST_QUERY);
    }

    public List<KeyValuePair> readCustomList(int i) throws SQLException {
        return readKeyValuePairList(CUSTOM_LIST_QUERY[i]);
    }

    public List<DosboxVersion> readDosboxVersionsList() throws SQLException {
        List<DosboxVersion> dosboxVersionsList = new ArrayList<DosboxVersion>();
        Statement stmt = null;
        ResultSet rs = null;
        try {
            stmt = con.createStatement();
            rs = stmt.executeQuery(DOSBOXVERSIONS_LIST_QUERY);
            if (rs != null)
                while (rs.next())
                    dosboxVersionsList.add(new DosboxVersion(rs.getInt(1), rs.getString(2), rs.getString(3),
                            rs.getBoolean(4), rs.getBoolean(5), rs.getString(6), rs.getString(7)));
        } catch (SQLException e) {
            e.printStackTrace();
            throw new SQLException("Database query 'read dosboxversions' failed");
        } finally {
            closeAll(stmt, rs);
        }
        return dosboxVersionsList;
    }

    public List<Template> readTemplatesList() throws SQLException {
        List<Template> templatesList = new ArrayList<Template>();
        Statement stmt = null;
        ResultSet rs = null;
        try {
            stmt = con.createStatement();
            rs = stmt.executeQuery(TEMPLATES_LIST_QUERY);
            if (rs != null)
                while (rs.next())
                    templatesList.add(new Template(rs.getInt(1), rs.getString(2), rs.getInt(3), rs.getBoolean(4)));
        } catch (SQLException e) {
            e.printStackTrace();
            throw new SQLException("Database query 'read templates' failed");
        } finally {
            closeAll(stmt, rs);
        }
        return templatesList;
    }

    public void removeProfile(int gameId) throws SQLException {
        removeSomething(REMOVE_GAME_QUERY, "profile", gameId);
    }

    public void removeDosboxVersion(int dbversionId) throws SQLException {
        removeSomething(REMOVE_DOSBOXVERSION_QUERY, "dosboxversion", dbversionId);
    }

    public void removeTemplate(int templateId) throws SQLException {
        removeSomething(REMOVE_TEMPLATE_QUERY, "template", templateId);
    }
    
    public int addOrEditProfile(String title, String developer, String publisher, String genre, String year,
            String status, String setup, String setupParams, String notes, boolean favorite, int devId, int publId,
            int genId, int yrId, int statId, int dbversionId, String[] link, String[] sCust, int[] iCust, int[] custId,
            int profileId) throws SQLException {
        int result = -1;
        PreparedStatement pstmt = null;
        try {
            if (devId == -1)    devId = addSomething(ADD_DEVELOPER_QUERY, developer);
            if (publId == -1)   publId = addSomething(ADD_PUBLISHER_QUERY, publisher);
            if (genId == -1)    genId = addSomething(ADD_GENRE_QUERY, genre);
            if (yrId == -1)     yrId = addSomething(ADD_YEAR_QUERY, year);
            if (statId == -1)   statId = addSomething(ADD_STATUS_QUERY, status);
            for (int i = 0; i < COMPLEX_CUSTOM_STRINGS; i++)
                if (custId[i] == -1) custId[i] = addSomething(ADD_CUSTOM_QUERY[i], sCust[i]);

            pstmt = con.prepareStatement((profileId == -1) ? ADD_GAME_QUERY : UPDATE_GAME_QUERY);
            pstmt.setString(1, title);
            pstmt.setInt(2, devId);
            pstmt.setInt(3, publId);
            pstmt.setInt(4, genId);
            pstmt.setInt(5, yrId);
            pstmt.setInt(6, dbversionId);
            pstmt.setInt(7, statId);
            pstmt.setString(8, setup);
            pstmt.setString(9, setupParams);
            pstmt.setString(10, notes);
            pstmt.setString(11, link[0]);
            pstmt.setString(12, link[1]);
            pstmt.setBoolean(13, favorite);
            pstmt.setString(14, link[2]);
            pstmt.setString(15, link[3]);
            for (int i = 0; i < COMPLEX_CUSTOM_STRINGS; i++)
                pstmt.setInt(i + 16, custId[i]);
            for (int i = 0; i < SIMPLE_CUSTOM_STRINGS; i++)
                pstmt.setString(i + COMPLEX_CUSTOM_STRINGS + 16, sCust[i + COMPLEX_CUSTOM_STRINGS]);
            for (int i = 0; i < SIMPLE_CUSTOM_INTS; i++)
                pstmt.setInt(i + COMPLEX_CUSTOM_STRINGS + SIMPLE_CUSTOM_STRINGS + 16, iCust[i]);

            if (profileId == -1) {
                // create new profile
                pstmt.executeUpdate();
                result = identity();
            } else {
                // update existing profile
                pstmt.setInt(COMPLEX_CUSTOM_STRINGS + SIMPLE_CUSTOM_STRINGS + SIMPLE_CUSTOM_INTS + 16, profileId);
                pstmt.executeUpdate();
                result = profileId;
            }
        } catch (SQLException e) {
            e.printStackTrace();
            throw new SQLException("Database query 'add/edit profile' failed");
        } finally {
            closeAll(pstmt, null);
        }
        return result;
    }

    public void updateProfileConf(String confFile, String captures, int profileId) throws SQLException {
        PreparedStatement stmt = null;
        try {
            stmt = con.prepareStatement(UPDATE_GAME_CONF_QUERY);
            stmt.setString(1, confFile);
            stmt.setString(2, captures);
            stmt.setInt(3, profileId);
            stmt.executeUpdate();
        } catch (SQLException e) {
            e.printStackTrace();
            throw new SQLException("Database query 'update profile conf' failed");
        } finally {
            closeAll(stmt, null);
        }
    }

    public void updateProfileSetupAndLinks(String setup, String[] links, int profileId) throws SQLException {
        PreparedStatement stmt = null;
        try {
            stmt = con.prepareStatement(UPDATE_GAME_SETUP_AND_LINKS_QUERY);
            stmt.setString(1, setup);
            stmt.setString(2, links[0]);
            stmt.setString(3, links[1]);
            stmt.setString(4, links[2]);
            stmt.setString(5, links[3]);
            stmt.setInt(6, profileId);
            stmt.executeUpdate();
        } catch (SQLException e) {
            e.printStackTrace();
            throw new SQLException("Database query 'update profile setup and links' failed");
        } finally {
            closeAll(stmt, null);
        }
    }

    public void toggleFavorite(int profileId) throws SQLException {
        PreparedStatement stmt = null;
        try {
            stmt = con.prepareStatement(TOGGLE_FAVORITE_QUERY);
            stmt.setInt(1, profileId);
            stmt.executeUpdate();
        } catch (SQLException e) {
            e.printStackTrace();
            throw new SQLException("Database query 'toggle favorite' failed");
        } finally {
            closeAll(stmt, null);
        }
    }

    public int duplicateProfile(int profileId) throws SQLException {
        int result = -1;
        PreparedStatement pstmt = null;
        try {
            pstmt = con.prepareStatement(DUPLICATE_GAME_QUERY);
            pstmt.setInt(1, profileId);
            pstmt.executeUpdate();
            result = identity();
        } catch (SQLException e) {
            e.printStackTrace();
            throw new SQLException("Database query 'duplicate profile' failed");
        } finally {
            closeAll(pstmt, null);
        }
        return result;
    }

    public int addOrEditDosboxVersion(String title, String path, boolean multiConfig, boolean defaultVersion,
            String parameters, String version, int dbversionId) throws SQLException {
        int result = -1;
        Statement stmt = null;
        PreparedStatement pstmt = null;
        try {
            if (defaultVersion) {
                stmt = con.createStatement();
                stmt.executeQuery(UPDATE_DOSBOXVERSION_NODEFAULT);
            }
            if (dbversionId == -1) {
                // create new dosbox version
                pstmt = con.prepareStatement(ADD_DOSBOXVERSION_QUERY);
                pstmt.setString(1, title);
                pstmt.setString(2, path);
                pstmt.setBoolean(3, multiConfig);
                pstmt.setBoolean(4, defaultVersion);
                pstmt.setString(5, parameters);
                pstmt.setString(6, version);
                pstmt.executeUpdate();
                result = identity();
            } else {
                // update existing dosbox version
                pstmt = con.prepareStatement(UPDATE_DOSBOXVERSION_QUERY);
                pstmt.setString(1, title);
                pstmt.setString(2, path);
                pstmt.setBoolean(3, multiConfig);
                pstmt.setBoolean(4, defaultVersion);
                pstmt.setString(5, parameters);
                pstmt.setString(6, version);
                pstmt.setInt(7, dbversionId);
                pstmt.executeUpdate();
                result = dbversionId;
            }
        } catch (SQLException e) {
            e.printStackTrace();
            throw new SQLException("Database query 'add/edit dosboxversion' failed");
        } finally {
            closeAll(stmt, null);
            closeAll(pstmt, null);
        }
        return result;
    }

    public int addOrEditTemplate(String title, int dbversionId, boolean defaultVersion, int templateId)
            throws SQLException {
        int result = -1;
        Statement stmt = null;
        PreparedStatement pstmt = null;
        try {
            if (defaultVersion) {
                stmt = con.createStatement();
                stmt.executeQuery(UPDATE_TEMPLATE_NODEFAULT);
            }
            if (templateId == -1) {
                // create new template
                pstmt = con.prepareStatement(ADD_TEMPLATE_QUERY);
                pstmt.setString(1, title);
                pstmt.setInt(2, dbversionId);
                pstmt.setBoolean(3, defaultVersion);
                pstmt.executeUpdate();
                result = identity();
            } else {
                // update existing template
                pstmt = con.prepareStatement(UPDATE_TEMPLATE_QUERY);
                pstmt.setString(1, title);
                pstmt.setInt(2, dbversionId);
                pstmt.setBoolean(3, defaultVersion);
                pstmt.setInt(4, templateId);
                pstmt.executeUpdate();
                result = templateId;
            }
        } catch (SQLException e) {
            e.printStackTrace();
            throw new SQLException("Database query 'add/edit template' failed");
        } finally {
            closeAll(stmt, null);
            closeAll(pstmt, null);
        }
        return result;
    }
}
